#!/usr/bin/python

# TODO: handle filename collisions when inserting into categories!
# (two different files with the same name, which happen to fit into
# the same category..  causes problems)
# example is people/hardwire/img_0400.jpg

import photos

def md5sum(path):
  import md5

  bufsize = 8192
  fd = open(path, "rb")
  m = md5.new()
  
  done = 0
  while not done:
    buf = fd.read(bufsize)
    if len(buf) == 0:
      done = 1
    else:
      m.update(buf)

  fd.close()
  
  result = m.hexdigest()
  return result

def getdate(path, date):
  # verify that the date given is valid...
  if date:
    import re
    pat = re.compile(r"(\d\d\d\d)-(\d\d)-(\d\d) (\d\d):(\d\d):(\d\d)")
    f = pat.search(date)
    if f:
      date = "%04i-%02i-%02i %02i:%02i:%02i" % (int(f.group(1)),
             int(f.group(2)), int(f.group(3)), int(f.group(4)),
	     int(f.group(5)), int(f.group(6)))
      return date
    else:
      raise ValueError, "Invalid date: %s" % (date)
  # otherwise, use the file's mtime
  else:
    import os
    import time

    s = os.stat(path)
    t = time.localtime(s.st_mtime)[:6]
    d = "%04i-%02i-%02i %02i:%02i:%02i" % t
    return d

def touch(path):
  import os.path
  if not os.path.exists(path):
    f = open(path, "w")
    f.close()

def unused_path(path, md5=None):
  #print "unused_path(%s, %s)" % (path, md5)
  import string
  import os.path
  import re

  print "checking... %s" % (path)

  # limit recursion
  if not os.path.exists(path):
    print "new entry: %s" % (path)
    return path

  # check if we've already renamed this file
  else:
    if md5:
      my_md5 = md5sum("%s/.data" % path)
      #print "md5: %s =?= %s" % (md5, my_md5)
      if my_md5 == md5:
	print "md5 matches: %s" % (path)
	return path
  
  d = os.path.dirname(path)
  f = os.path.basename(path)
  #print "dir: %s\nbase: %s" % (d, f)

  # file.23.ext -> file.24.ext
  pat = re.compile(r"^(.*)\.(\d+)\.([^\.]*)$")
  found = pat.search(f)
  if found:
    n = "%s.%s.%s" % (found.group(1), int(found.group(2))+1, found.group(3))
    return unused_path("%s/%s" % (d, n), md5)

  # file.23 -> file.24
  pat = re.compile(r"^(.*)\.(\d+)$")
  found = pat.search(f)
  if found:
    n = "%s.%s" % (found.group(1), int(found.group(2))+1)
    return unused_path("%s/%s" % (d, n), md5)

  # file.ext -> file.1.ext
  pat = re.compile(r"^(.*)\.([^\.]*)$")
  found = pat.search(f)
  if found:
    n = "%s.1.%s" % (found.group(1), found.group(2))
    return unused_path("%s/%s" % (d, n), md5)

  # if no other patterns match...
  # file -> file.1
  f = f + ".1"
  return unused_path("%s/%s" % (d, f), md5)

class empty:
  pass

def run(args):
  import os.path
  import mimetypes

  cats = []
  files = []
  anonread = 1
  anonwrite = 0
  readers = []
  writers = []
  desc = ""
  date = None
  dategiven = 0

  i = 0
  while i < len(args):
    a = args[i]
    #print "Processing: %s" % (a)

    if a in ["-c", "--category"]:
      i = i + 1
      c = args[i]
      cats.append(c)

    elif a in ["-d", "--desc"]:
      i = i + 1
      d = args[i]
      desc = d

    elif a in ["-t", "--date"]:
      i = i + 1
      d = args[i]
      date = d
      dategiven = 1

    elif a in ["-a", "--anonread"]:
      i = i + 1
      a = args[i]
      if a in ["0", "false", "no"]:
	anonread = 0
      elif a in ["1", "true", "yes"]:
	anonread = 1
      else:
	print "Invalid anonread value: %s" % (a)
	return

    elif a in ["-W", "--anonwrite"]:
      i = i + 1
      a = args[i]
      if a in ["0", "false", "no"]:
	anonwrite = 0
      elif a in ["1", "true", "yes"]:
	anonwrite = 1
      else:
	print "Invalid anonwrite value: %s" % (a)
	return

    elif a in ["-r", "--read"]:
      i = i + 1
      u = args[i]
      readers.append(u)

    elif a in ["-w", "--write"]:
      i = i + 1
      u = args[i]
      writers.append(u)

    elif a in ["-h", "-?", "--help"]:
      print "Photo insertion script."
      print "Summary:"
      print "  Inserts files into your photo database, sets up permissions, and"
      print "  enters photos into categories."
      print "Usage:"
      print "\tinsert.py [options] [files]"
      print "Options:"
      print "  -h     --help       This help text."
      print "  -c arg --category   Add files to this category (may be repeated)"
      print "  -d arg --desc       Set a description for the files."
      print "  -t arg --date       Override the date for the files. (YYYY-MM-DD HH:MM:SS)"
      print "  -a arg --anonread   Enable/disable public read access to files.  [def 1]"
      print "  -W arg --anonwrite  Enable/disable public write access to files. [def 0]"
      print "  -r arg --read       Add user to read access list (repeatable)"
      print "  -w arg --write      Add user to write access list (repeatable)"
      return

    else:
      f = empty()
      f.path = a
      f.base = os.path.basename(f.path)
      f.md5 = md5sum(f.path)
      f.newpath = "%s/flat/%s/%s.%s" % (photos.root, f.md5[:2], f.md5, f.base)
      f.date = getdate(f.path, date)
      #print "%s -> %s" % (f.path, f.newpath)
      files.append(f)

    i = i + 1


  # do the actual work
  # create the categories if they don't exist
  for c in cats:
    cpath = "%s/cat/%s" % (photos.root, c)

    # TODO: remember to create perms recursively for new cats,
    #       if the parents don't yet exist!
    #       ... or maybe not.  This should perhaps be an error the user
    #       should know about, and handle.
    if not os.path.exists(cpath):
      print "creating %s" % (cpath)
      #os.makedirs(cpath)
      os.mkdir(cpath)
      
    if not os.path.exists("%s/.mimetype" % (cpath)):
      print "creating %s" % ("%s/.mimetype" % (cpath))
      f = open("%s/.mimetype" % cpath, "w")
      f.write("inode/directory\n")
      f.close()

    if not files:
      #if desc  and  not os.path.exists("%s/.description" % (cpath)):
      # Why did I not overwrite before?
      if desc:
	print "creating %s/.description" % (cpath)
	f = open("%s/.description" % cpath, "w")
	f.write("%s\n" % (desc))
	f.close()

      if anonread  and  not os.path.exists("%s/.anonread" % (cpath)):
	print "touch %s/.anonread" % (cpath)
	# TODO: add .anonread removal when "-a 0" is specified
	touch("%s/.anonread" % (cpath))

      if anonwrite  and  not os.path.exists("%s/.anonwrite" % (cpath)):
	print "touch %s/.anonwrite" % (cpath)
	touch("%s/.anonwrite" % (cpath))

      for u in readers:
	if not os.path.exists("%s/.read" % (cpath)):
	  os.mkdir("%s/.read" % (cpath))
	if not os.path.exists("%s/.read/%s" % (cpath, u)):
	  print "touch %s/.read/%s" % (cpath, u)
	  touch("%s/.read/%s" % (cpath, u))

      for u in writers:
	if not os.path.exists("%s/.write" % (cpath)):
	  os.mkdir("%s/.write" % (cpath))
	if not os.path.exists("%s/.write/%s" % (cpath, u)):
	  print "touch %s/.write/%s" % (cpath, u)
	  touch("%s/.write/%s" % (cpath, u))

  print ""

  # enter the files into the db, and link them into the categories
  for f in files:
    exists = 0
    # Create the file directory...
    if os.path.exists(f.newpath):
      print "File already in database; updating."
      exists = 1
    else:
      os.makedirs(f.newpath)

    # move or link files into db...
    if not exists:
      print "%s -> %s/.data" % (f.path, f.newpath)
      try:
	os.link(f.path, "%s/.data" % (f.newpath))
      except Exception, msg:
	print "Couldn't link file: %s" % (msg)
	# try to copy instead of link if the file isn't already there
	import string
	if string.find("%s" % (msg), "exists") < 0:
	  import shutil
	  shutil.copy(f.path, "%s/.data" % (f.newpath))

    # create mimetype...
    if not os.path.exists("%s/.mimetype" % (f.newpath)):
      mt = mimetypes.guess_type(f.path)
      if mt[0]:
	print "creating %s/.mimetype (%s)" % (f.newpath, mt[0])
	fp = open("%s/.mimetype" % (f.newpath), "w")
	fp.write(mt[0])
	fp.write("\n")
	fp.close()
      else:
	print "unable to guess mimetype"

    # create permissions...
    if anonread:
      print "touch %s/.anonread" % (f.newpath)
      # TODO: add .anonread removal when "-a 0" is specified
      touch("%s/.anonread" % (f.newpath))
    if anonwrite:
      print "touch %s/.anonwrite" % (f.newpath)
      touch("%s/.anonwrite" % (f.newpath))
    for u in readers:
      if not os.path.exists("%s/.read" % (f.newpath)):
	os.mkdir("%s/.read" % (f.newpath))
      print "touch %s/.read/%s" % (f.newpath, u)
      touch("%s/.read/%s" % (f.newpath, u))
    for u in writers:
      if not os.path.exists("%s/.write" % (f.newpath)):
	os.mkdir("%s/.write" % (f.newpath))
      print "touch %s/.write/%s" % (f.newpath, u)
      touch("%s/.write/%s" % (f.newpath, u))

    # handle other metadata...
    # TODO: create .mime-type file
    if desc:
      print "creating %s/.description" % (f.newpath)
      fp = open("%s/.description" % (f.newpath), "w")
      fp.write(desc)
      fp.write("\n")
      fp.close()

    if f.date:
      # don't auto-reset the date if it was already there
      if dategiven  or  not os.path.exists("%s/.date" % (f.newpath)):
	print "set %s/.date to '%s'" % (f.newpath, f.date)
	fp = open("%s/.date" % (f.newpath), "w")
	fp.write(f.date)
	fp.write("\n")
	fp.close()
    
    # insert into categories...
    for c in cats:
      cpath = "%s/cat/%s/%s" % (photos.root, c, f.base)
      # check checksum to make sure it's identical
      # change filename if not
      cpath = unused_path(cpath, f.md5)
      if not os.path.exists(cpath):
	print "%s -> %s" % (cpath, f.newpath)
	os.symlink(f.newpath, cpath)

      # Add the cached cat entry for the "also in" feature
      dotted = c.replace("/", ".")
      if dotted[:1] == ".": dotted = dotted[1:]
      dotpath = "%s/.cats/%s" % (f.newpath, dotted)
      contents = c
      if contents[:1] != "/":  contents = "/%s" % (contents)
      print "creating %s (%s)" % (dotpath, contents)
      if not os.path.exists("%s/.cats" % (f.newpath)):
	os.mkdir("%s/.cats" % (f.newpath))
      fp = open(dotpath, "w")
      fp.write("%s\n" % (contents))
      fp.close()
      

    print ""

if __name__ == "__main__":
  import sys
  run(sys.argv[1:])
