<?php
include_once("dirfiles.inc");

/*
  User login support...
  Cases:
    - anonymous user (show anonymous, generic menu)
      - user has access
        - display requested content
      - login required
        - display login page, keep track of where the user was
    - logged-in user (show context-sensitive menu)
      - user has access
        - display requested content
      - user denied access
        - display access denied message
*/

session_start();
session_register("session");

check_timeout();
//if ($action == "logout") { logout(); return; }

function check_timeout()
{
  if($session)
  {
    $timeout = 60*60*2;  // 120 minutes
    $curtime = time();
     
    if($session->lastviewtime)
    {
      $elapsed = $curtime - $session->lastviewtime;
      if($elapsed > $timeout)
      {
	$action = "expire";
	logout();
	return;
      }
      else { $session->lastviewtime = $curtime; }
    }
    else { $session->lastviewtime = time(); }
  }
}

function logout()
{
  global $session;
  global $HTTP_POST_VARS;
  global $HTTP_GET_VARS;

  session_destroy();
  $p = session_get_cookie_params();
  setcookie(session_name(), "", 0, $p["path"], $p["domain"]);
  $session = NULL;

  $HTTP_POST_VARS = NULL;
  $HTTP_GET_VARS = NULL;

  //$action = NULL;
  //printf("You have been logged out.\n");
}

function check_login() 
{
  global $session;
  return (!(!($session->user)));
}

function login_form()
{
  global $HTTP_GET_VARS;
  //global $PHP_SELF;
  global $photourl;
  global $path;

  print "Log in:\n";

  //print "(Forgot your password?)<br>\n";
  //page("forgot_password.php", "password");
  print "<div class=\"login_form\">\n";
  print "  <form method=\"post\" action=\"{$photourl}/_login/{$path}\">\n";
  print "    Username: <input type=\"text\" name=\"user\"><p>\n";
  print "    Password: <input type=\"password\" name=\"passwd\"><p>\n";
  print "    <input type=\"submit\" value=\"Go\">\n";
  print "    <input type=\"submit\" value=\"Nevermind...\">\n";
  print "  </form>\n";
  print "</div>\n";
}

function verify_user($name, $passwd)
{
  global $photoroot;

  //print "verify_user({$name}, {$passwd})<br>\n";
  // Check if user is valid.
  // If so, return their authenticated user ID.
  $found_user = "";

  $upath = "{$photoroot}/users/{$name}";

  if (! file_exists($upath))
  {
    //print "No user found.<br>\n";
    return 0;
  }
  
  $found_user = dirfile($upath);
  $found_user->name = $name;
  $found_user->read_attrib(".password");
  $found_user->read_attrib(".realname");

  if(! $found_user->attribs[".password"]) return 0;
  if(! $found_user->attribs[".realname"]) return 0;

  if(! ($passwd == trim($found_user->attribs[".password"]))) return 0;

  return $found_user;
}

function auth_read($path, $user)
{
   global $photoroot;
   global $anonread;
   global $readacl;

   $parts = explode("/", $path);
   $priv = "";
   if($user != "")
   {
      $priv = "{$readacl}/{$user}";
   }

   // Check the path perms, from $photoroot to $photoroot/$path,
   // making sure each level is authorized.
   $check = $photoroot;
   $done = 0;
   while(! $done)
   {
      //print "Checking...  {$check}\n";
      // check current path...
      $approved = 0;
      if(file_exists("{$check}/{$anonread}")) {$approved = 1; }
      elseif($user != "")
      {
	 if(file_exists("{$check}/{$priv}")) {$approved = 1; }
      }
      // if not approved, return false
      if(!$approved) return 0;
      //else {print "... okay<br>\n";}
      // if no parts left, return true
      $s = sizeof($parts);
      /*
      print "Remaining: {$s} (\n";
      foreach ($parts as $p)
      {
	 print " / {$p}\n";
      }
      print ")<br>\n";
      */
      if(sizeof($parts) == 0) return 1;
      // add to current path...
      $check = $check . "/" . array_shift($parts);
   }
   return 0;
}

function read_dir($path, $user)
{
   // return a list of files, where each file is a dirfile object,
   // with its mimetype, basename, wwwpath?, and fullpath? filled in...
   // (other data must be loaded explicitly)
   // don't return *real* files; only dirfiles
   global $photoroot;
   global $anonread;
   global $readacl;

   // check if dir is readable
   $allowed = auth_read($path, $user);
   //print "auth_read({$path}, {$user}) => {$allowed}<br>\n";
   if(!$allowed) { return -1; }

   // read contents of dir
   $allfiles = get_dir_array($path);

   // TODO: filter out dotfiles

   // filter contents by permissions
   $result = array();
   if($user == "")
   {
      // return all public entries
      foreach ($allfiles as $f)
      {
	 $fpath = "{$photoroot}/{$path}/{$f}";
	 //print "Check access for {$f}...\n";
	 $p = "{$photoroot}/{$path}/{$f}/{$anonread}";
	 //print "{$p}\n";
	 if(file_exists($p)) { 
	    $df = dirfile($fpath);
	    $df->anonread = 1;
	    $df->read_mime();
	    $result[] = $df; 
	    //print "okay. ({$p})\n";
	 }
	 //print "<br>\n";
      }
   }
   else
   {
      // return all public and authorized private entries
      $priv = "{$readacl}/{$user}";
      foreach ($allfiles as $f)
      {
	 $fpath = "{$photoroot}/{$path}/{$f}";
	 $p = "{$photoroot}/{$path}/{$f}/{$anonread}";
	 $p2 = "{$photoroot}/{$path}/{$f}/{$priv}";
	 if(file_exists($p)) 
	 { 
	   $df = dirfile($fpath);
	   $df->anonread = 1;
	   $df->read_mime();
	   $result[] = $df; 
	 }
	 elseif(file_exists($p2)) 
	 { 
	   $df = dirfile($fpath);
	   $df->anonread = 0;
	   $df->read_mime();
	   $result[] = $df; 
	 }
      }
   }
   return $result;
}
?>
